# Copyright (C) 2006-2014 Oracle. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, version 2.  This program is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the GNU
# General Public License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 021110-1307, USA.

from xml.dom.minidom import Document


class SimpleDiscoveryXML:
    """
    A simple XML document of the form needed for discovery results.

    Provides for pushing (nesting) elements and adding elements with a
    single value.  The document can be formatted as a pretty-printed
    string.
    All errors are reported by raising exceptions.
    """
    def __init__(self, toplevel):
        # Create a XML document starting with the specified element
        if not toplevel:
            raise Exception("A top level element name is required")
        self.document = Document()
        self.elementstack = []
        top = self.document.createElement(toplevel)
        self.elementstack.append(top)
        self.document.appendChild(top)

    def add(self, element, value=None, attribute=None, attr_value=None):
        elem = self.document.createElement(element)
        top = self.elementstack[-1]
        top.appendChild(elem)
        if value != None:
            elem.appendChild(self.document.createTextNode(value))
        if attribute:
            elem.setAttribute(attribute, attr_value)

    def push(self, element, *attributes):
        newtop = self.document.createElement(element)
        oldtop = self.elementstack[-1]
        self.elementstack.append(newtop)
        oldtop.appendChild(newtop)
        for index in range(0, len(attributes), 2):
            newtop.setAttribute(attributes[index], attributes[index + 1])

    def pop(self):
        if len(self.elementstack) == 0:
            raise Exception("Popped empty element stack in SimpleDiscoveryXML")
        self.elementstack.pop()

    def finish(self):
        if len(self.elementstack) > 1:
            raise Exception("Unpopped element in SimpleDiscoveryXML")
        result = self.document.toxml()
        return result
