# Copyright (C) 2006-2014 Oracle. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, version 2.  This program is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the GNU
# General Public License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 021110-1307, USA.

import logging
import os
import pickle
import socket
import struct

from agent.lib.constants import NOTIFICATION_SERVER_ADDRESS


def send_message(message_type, message, data=None):
    data = data or {}
    session = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
    session.connect(NOTIFICATION_SERVER_ADDRESS)
    try:
        msg = pickle.dumps((message_type, message, data))
        msglen = struct.pack(">L", len(msg))
        session.sendall(msglen + msg)
    except Exception, err:
        logging.error("Error sending message: %s", err)
        raise err
    finally:
        try:
            session.close()
        except Exception:
            pass


def notify_manager(event, data=None):
    data = data or {}
    return send_message('event', event, data)


def send_statistic(statistic):
    send_message('statistic', statistic)


def async_proc_prog(status):
    event = '{ASYNC_PROC} progress PID %s' % (os.getpid(),)
    try:
        notify_manager(event, status)
    except Exception, err:
        logging.warn('Error sending event: %s status: %s error: %s',
                     event, status, err)
