#!/usr/bin/env python
#
# Copyright (C) 2011-2014 Oracle. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, version 2.  This program is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the GNU
# General Public License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 021110-1307, USA.
####################################################################################################
#
#	Author:		Joshua Nicholas
#
"""Utilities for dealing with generic devices
"""

import st_utils

import os
import re
import traceback

####################################################################################################

__all__ = [	'get_device_major_nums'
		,	'get_character_device_major_num'
		,	'get_block_device_major_num' ]

####################################################################################################

global__cached__char_device_major_nums_LIST_DICT	= None
global__cached__blck_device_major_nums_LIST_DICT	= None

####################################################################################################
def get_device_major_nums( vi_errorAndLog_obj, refresh ):
	"""
	This routine
	"""
#
	global global__cached__char_device_major_nums_LIST_DICT
	global global__cached__blck_device_major_nums_LIST_DICT
	if not refresh and ( global__cached__char_device_major_nums_LIST_DICT or global__cached__blck_device_major_nums_LIST_DICT ):
		return ( global__cached__char_device_major_nums_LIST_DICT, global__cached__blck_device_major_nums_LIST_DICT )

	char_device_major_nums_LIST_DICT = {}
	blck_device_major_nums_LIST_DICT = {}

	proc_devices_lines_LIST, read_failed = st_utils.vi_read_text_lines( vi_errorAndLog_obj, '/proc/devices' )
	if read_failed:
		vi_errorAndLog_obj.vi_croak( 'read of /proc/devices failed !?!' )
		return ( None, None )

	maj_finder_cmpldRegEx				= re.compile( '\s*(\d+)\s+(\S+)\s*$' )
	char_dev_start_marker_cmpldRegEx	= re.compile( '^Character devices:$' )
	blck_dev_start_marker_cmpldRegEx	= re.compile( '^Block devices:$' )

	for idx in range( 0, len(proc_devices_lines_LIST) ):
		proc_devices_line	= proc_devices_lines_LIST[ idx ]
		if not char_dev_start_marker_cmpldRegEx.match( proc_devices_line ):
			continue
		idx	+= 1
		break

	for idx in range( idx, len(proc_devices_lines_LIST) ):
		proc_devices_line	= proc_devices_lines_LIST[ idx ]
		if not blck_dev_start_marker_cmpldRegEx.match( proc_devices_line ):
			maj_finder_match = maj_finder_cmpldRegEx.match( proc_devices_line )
			if maj_finder_match:
				device_name				= maj_finder_match.group(2)
				device_major_num		= int( maj_finder_match.group(1) )
				device_major_nums_LIST	= char_device_major_nums_LIST_DICT.get( device_name, None )
				if not device_major_nums_LIST:
					char_device_major_nums_LIST_DICT[ device_name ]	= [ device_major_num ]
				else:
					device_major_nums_LIST.append( device_major_num )
				continue
			if len( proc_devices_line ) < 1:
				continue
			vi_errorAndLog_obj.vi_print_err( "BAD /proc/devices line '%s'" % ( proc_devices_line ) )
			continue
		idx	+= 1
		break

	for idx in range( idx, len(proc_devices_lines_LIST) ):
		proc_devices_line	= proc_devices_lines_LIST[ idx ]
		maj_finder_match = maj_finder_cmpldRegEx.match( proc_devices_line )
		if maj_finder_match:
			device_name				= maj_finder_match.group(2)
			device_major_num		= int( maj_finder_match.group(1) )
			device_major_nums_LIST	= blck_device_major_nums_LIST_DICT.get( device_name, None )
			if not device_major_nums_LIST:
				blck_device_major_nums_LIST_DICT[ device_name ]	= [ device_major_num ]
			else:
				device_major_nums_LIST.append( device_major_num )
			continue
		if len( proc_devices_line ) < 1:
			continue
		vi_errorAndLog_obj.vi_print_err( "BAD /proc/devices line '%s'" % ( proc_devices_line ) )

	global__cached__char_device_major_nums_LIST_DICT	= char_device_major_nums_LIST_DICT
	global__cached__blck_device_major_nums_LIST_DICT	= blck_device_major_nums_LIST_DICT
	return ( global__cached__char_device_major_nums_LIST_DICT, global__cached__blck_device_major_nums_LIST_DICT )

####################################################################################################
def get_character_device_major_num( vi_errorAndLog_obj, deviceName, refresh ):
	"""
	This routine
	"""
#
	char_device_major_nums_LIST_DICT, blck_device_major_nums_LIST_DICT	= get_device_major_nums( vi_errorAndLog_obj, refresh )

	if not char_device_major_nums_LIST_DICT:
		return -1

	device_major_nums_LIST	= char_device_major_nums_LIST_DICT.get( deviceName, None )
	if not device_major_nums_LIST:
		return -2

	if len( device_major_nums_LIST ) != 1:
		return -3

	return device_major_nums_LIST[ 0 ]

####################################################################################################
def get_block_device_major_num( vi_errorAndLog_obj, deviceName, refresh ):
	"""
	This routine
	"""
#
	char_device_major_nums_LIST_DICT, blck_device_major_nums_LIST_DICT	= get_device_major_nums( vi_errorAndLog_obj, refresh )

	if not char_device_major_nums_LIST_DICT:
		return -1

	device_major_nums_LIST	= blck_device_major_nums_LIST_DICT.get( deviceName, None )
	if not device_major_nums_LIST:
		return -2

	if len( device_major_nums_LIST ) != 1:
		return -3

	return device_major_nums_LIST[ 0 ]

####################################################################################################
# END
