#!/usr/bin/env python
#
# Copyright (C) 2011-2014 Oracle. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, version 2.  This program is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the GNU
# General Public License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 021110-1307, USA.
####################################################################################################
#
#	Author:		Joshua Nicholas
#
"""Utilities for byte calculations and such
"""

####################################################################################################

__all__ = [	'vi_align_to_next'
		,	'vi_align_to_prev'
		,	'vi_byteStr4ToUInt32'
		,	'vi_byteStr8ToUInt64'
		,	'vi_as_hex' ]

####################################################################################################
def vi_align_to_next( valueToBeAligned, alignmentCriteria ):
	"""
	This routine
	"""
#
	if valueToBeAligned < 0:
		raise ValueError, "valueToBeAligned %s is negative [ %s ]" % ( valueToBeAligned, alignmentCriteria )

	if alignmentCriteria < 0:
		return 0

	if alignmentCriteria == 0 or alignmentCriteria == 1:
		return valueToBeAligned

	alignedValueToReturn	 = valueToBeAligned + alignmentCriteria - 1

	if alignedValueToReturn < valueToBeAligned:
		if valueToBeAligned % alignmentCriteria == 0:
			return valueToBeAligned
		raise OverflowError, "%s precision exceeded [ %s %s ]" % ( type(valueToBeAligned), valueToBeAligned, alignmentCriteria )

	alignedValueToReturn	-= ( alignedValueToReturn % alignmentCriteria )

	return alignedValueToReturn

####################################################################################################
def vi_align_to_prev( valueToBeAligned, alignmentCriteria ):
	"""
	This routine
	"""
#
	if valueToBeAligned < 0:
		raise ValueError, "valueToBeAligned %s is negative [ %s ]" % ( valueToBeAligned, alignmentCriteria )

	if alignmentCriteria < 0:
		return 0

	if alignmentCriteria == 0 or alignmentCriteria == 1:
		return valueToBeAligned

	alignedValueToReturn	= valueToBeAligned - ( valueToBeAligned % alignmentCriteria )

	return	alignedValueToReturn

####################################################################################################
def vi_byteStr4ToUInt32( byteStr4, isLittleEndianVSbigEndian ):
	"""
	This routine
	"""
#
	uint32_retVal	= 0

	if isLittleEndianVSbigEndian:
		uint32_retVal	= uint32_retVal | ( ord( byteStr4[ 0 ] ) << 24 )
		uint32_retVal	= uint32_retVal | ( ord( byteStr4[ 1 ] ) << 16 )
		uint32_retVal	= uint32_retVal | ( ord( byteStr4[ 2 ] ) << 8 )
		uint32_retVal	= uint32_retVal | ( ord( byteStr4[ 3 ] ) )
	else:
		uint32_retVal	= uint32_retVal | ( ord( byteStr4[ 0 ] ) )
		uint32_retVal	= uint32_retVal | ( ord( byteStr4[ 1 ] ) << 8 )
		uint32_retVal	= uint32_retVal | ( ord( byteStr4[ 2 ] ) << 16 )
		uint32_retVal	= uint32_retVal | ( ord( byteStr4[ 3 ] ) << 24 )

	return uint32_retVal

####################################################################################################
def vi_byteStr8ToUInt64( byteStr8, isLittleEndianVSbigEndian ):
	"""
	This routine
	"""
#
	uint64_retVal	= 0L

	if isLittleEndianVSbigEndian:
		uint64_retVal	= uint64_retVal | ( ord( byteStr8[ 0 ] ) << 56 )
		uint64_retVal	= uint64_retVal | ( ord( byteStr8[ 1 ] ) << 48 )
		uint64_retVal	= uint64_retVal | ( ord( byteStr8[ 2 ] ) << 40 )
		uint64_retVal	= uint64_retVal | ( ord( byteStr8[ 3 ] ) << 32 )
		uint64_retVal	= uint64_retVal | ( ord( byteStr8[ 4 ] ) << 24 )
		uint64_retVal	= uint64_retVal | ( ord( byteStr8[ 5 ] ) << 16 )
		uint64_retVal	= uint64_retVal | ( ord( byteStr8[ 6 ] ) << 8 )
		uint64_retVal	= uint64_retVal | ( ord( byteStr8[ 7 ] ) )
	else:
		uint64_retVal	= uint64_retVal | ( ord( byteStr8[ 0 ] ) )
		uint64_retVal	= uint64_retVal | ( ord( byteStr8[ 1 ] ) << 8 )
		uint64_retVal	= uint64_retVal | ( ord( byteStr8[ 2 ] ) << 16 )
		uint64_retVal	= uint64_retVal | ( ord( byteStr8[ 3 ] ) << 24 )
		uint64_retVal	= uint64_retVal | ( ord( byteStr8[ 4 ] ) << 32 )
		uint64_retVal	= uint64_retVal | ( ord( byteStr8[ 5 ] ) << 40 )
		uint64_retVal	= uint64_retVal | ( ord( byteStr8[ 6 ] ) << 48 )
		uint64_retVal	= uint64_retVal | ( ord( byteStr8[ 7 ] ) << 56 )

	return uint64_retVal

####################################################################################################
def vi_as_hex( givenBinaryBuffer ):

	outputHexStr = ''
	for byteAsChr in givenBinaryBuffer:
		byteAsInt = ord( byteAsChr )
		byteAsHex = hex( byteAsInt )[ 2 : ]
		if byteAsInt < 0x0F:
			byteAsHex = '0' + byteAsHex
		outputHexStr = outputHexStr + byteAsHex

	return outputHexStr.upper()

####################################################################################################
# END
